import 'dart:async';

import 'package:get/get.dart';
import 'package:motor_flutter/motor_flutter.dart';

const kBlockchainRestApiUri = "https://v1-beta.sonr.ws/sonr-io/sonr";

/// {@category Start Here}
/// ## QueryService
///
/// This service is responsible for querying the blockchain for structured data. It interacts with the
/// [WhatIs], [WhereIs], and [WhoIs] modules on chain.
class QueryService extends GetConnect {
  @override
  void onInit() {
    baseUrl = kBlockchainRestApiUri;
    super.onInit();
  }

  /// ### Get a User's WhoIs
  ///
  /// This method queries the blockchain for a user's WhoIs data. This contains the user's [DIDDocument] and
  /// [Alias] data.
  ///
  /// #### Parameters
  /// * `did` - The user's DID to query for
  Future<WhoIs?> whoIs(String did) async {
    try {
      final response = await httpClient.get<QueryWhoIsResponse>('/registry/who_is/$did', contentType: "application/json");
      if (!response.isOk || response.body == null) {
        return null;
      }
      return response.body?.whoIs;
    } catch (e) {
      Log.warn("Failed to call query using REST API on $baseUrl/$query");
    }
    return null;
  }

  /// ### Get a User's WhoIs by Alias
  ///
  /// This method queries the blockchain for a user's WhoIs data using a provided [alias].
  Future<WhoIs?> whoIsByAlias(String alias) async {
    try {
      final response = await httpClient.get<QueryWhoIsResponse>('/registry/who_is_alias/$alias', contentType: "application/json");
      if (!response.isOk || response.body == null) {
        return null;
      }
      return response.body?.whoIs;
    } catch (e) {
      Log.warn("Failed to call query using REST API on $baseUrl/$query");
    }
    return null;
  }

  /// ### Get a User's WhoIs by Controller
  ///
  /// This method queries the blockchain for a user's WhoIs data using a provided [controller].
  Future<WhoIs?> whoIsByController(String controller) async {
    try {
      final response = await httpClient.get<QueryWhoIsResponse>('/registry/who_is_controller/$controller', contentType: "application/json");
      if (!response.isOk || response.body == null) {
        return null;
      }
      return response.body?.whoIs;
    } catch (e) {
      Log.warn("Failed to call query using REST API on $baseUrl/$query");
    }
    return null;
  }

  /// ### Find a WhereIs
  ///
  /// This method queries the blockchain for a specific [WhereIs] by [did]. This contains a [Bucket] which
  /// stores individual user data.
  Future<WhereIs?> whereIs(String did) async {
    final response = await get("/bucket/whereis/$did");
    if (response.status.hasError) {
      return null;
    }
    return WhereIs.fromJson(response.body);
  }

  /// ### Get all WhereIs for a User
  ///
  /// This method queries the blockchain for a [List<WhereIs>] by [creator] and [did]. This contains a [Bucket] which
  /// stores individual user data. This method is useful for retreiving a list of applications a user has authorized.
  ///
  /// #### Parameters
  /// * `creator` - The user's DID to query for
  /// * `did` - The user's DID to query for
  Future<List<WhereIs>?> whereIsForCreatorDid(String creator, String did) async {
    try {
      final response = await httpClient.get<QueryWhereIsByCreatorResponse>('/bucket/whereis/$creator/$did', contentType: "application/json");
      if (!response.isOk || response.body == null) {
        return null;
      }
      return response.body?.whereIs;
    } catch (e) {
      Log.warn("Failed to call query using REST API on $baseUrl/$query");
    }
    return null;
  }

  /// ### Find a WhatIs
  ///
  /// This method queries the blockchain for a specific [WhatIs] by [did] and [creator]. This contains a [Schema] which
  /// defines how data is modeled on the Sonr Network.
  Future<WhatIs?> whatIs(String did, String creator) async {
    try {
      final response = await httpClient.get<QueryWhatIsResponse>('/schema/query/what_is?creator=$creator&did=$did', contentType: "application/json");
      if (!response.isOk || response.body == null) {
        return null;
      }
      return response.body?.whatIs;
    } catch (e) {
      Log.warn("Failed to call query using REST API on $baseUrl/$query");
    }
    return null;
  }

  /// ### Find a WhatIs by Creator
  ///
  /// This method queries the blockchain for a [List<WhatIs>] by [creator]. This contains a [Schema] which
  /// defines how data is modeled on the Sonr Network.
  Future<List<WhatIs>?> whatIsByCreator(String creator) async {
    try {
      final response = await httpClient.get<QueryWhatIsByCreatorResponse>('/schema/query/what_is/$creator', contentType: "application/json");
      if (!response.isOk || response.body == null) {
        return null;
      }
      return response.body?.whatIs;
    } catch (e) {
      Log.warn("Failed to call query using REST API on $baseUrl/$query");
    }
    return null;
  }
}
