import 'dart:async';

import 'package:flutter/material.dart';
import 'package:motor_flutter/motor_flutter.dart';
import 'package:motor_flutter/src/widgets/auth/pages/login_page.dart';

/// {@category Widgets}
///
/// The Sonr Login Button provides a Widget that can be used to trigger a dialog which
/// walks the user through the login process. Underneath this method
/// calls [MotorFlutter.login].
///
/// ### Parameters
/// - [variant] - The variant of the button to display. Defaults to [ButtonVariant.blue].
/// - [onSuccess] - A callback that is called when user login is successfull [LoginResponse].
/// - [onError] - A callback that is called when the user encounters an error while logging in.
///
/// ### Example
///
/// ```dart
/// import 'package:flutter/material.dart';
/// import 'package:motor_flutter/motor_flutter.dart';
///
/// class LoginWidget extends StatelessWidget {
///   const LoginWidget({Key? key}) : super(key: key);
///
///   @override
///   Widget build(BuildContext context) {
///     return Scaffold(
///       body: Center(
///         child: LoginButton(
///           variant: ButtonVariant.white,
///           onSuccess: (response) {
///             if (response != null) {
///               print('User successfully logged in!');
///              }
///            },
///           onError: (error) {
///            print('Error logging in user: $error');
///         },
///       ),
///     ),
///   );
/// }
/// ```
///
/// ## Options
///
/// ### [ButtonVariant.blue]
/// <img alt='Motor Button Preview' src='https://imagedelivery.net/B-Y55OAFnxqdmPWhf8E2OA/cca8ebae-3527-4b6c-e7c5-3180d4465b00/motorFlutterButtonPreviews'>
///
/// ### [ButtonVariant.black]
/// <img alt='Motor Button Preview' src='https://imagedelivery.net/B-Y55OAFnxqdmPWhf8E2OA/4895f4cc-36aa-46e5-b887-8702bf7f7100/motorFlutterButtonPreviews'>
///
/// ### [ButtonVariant.white]
/// <img alt='Motor Button Preview' src='https://imagedelivery.net/B-Y55OAFnxqdmPWhf8E2OA/61b3f3ca-623d-4586-a815-f5ad3b9fad00/motorFlutterButtonPreviews'>
///
/// ### [ButtonVariant.blackOutline]
/// <img alt='Motor Button Preview' src='https://imagedelivery.net/B-Y55OAFnxqdmPWhf8E2OA/cca8ebae-3527-4b6c-e7c5-3180d4465b00/motorFlutterButtonPreviews'>
///
/// ### [ButtonVariant.whiteOutline]
/// <img alt='Motor Button Preview' src='https://imagedelivery.net/B-Y55OAFnxqdmPWhf8E2OA/c20af003-5886-42fa-e255-2b5631448400/motorFlutterButtonPreviews'>
class LoginButton extends StatelessWidget {
  final ButtonVariant variant;
  final ResponseCallback<WhoIs>? onSuccess;
  final ErrorCallback? onError;

  const LoginButton(
      {Key? key,
      this.variant = ButtonVariant.blue,
      this.onSuccess,
      this.onError})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Material(
      child: Container(
          child: SonrButton(
        withLogo: false,
        variant: variant,
        label: 'Connect with Sonr',
        onPressed: () async {
          if (MotorFlutter.isReady) {
            final resp = await showDialog<WhoIs>(
              context: context,
              builder: (BuildContext context) {
                final completer = Completer<WhoIs>();
                return LoginPage(
                  onSuccess: (ai) {
                    Future.delayed(Duration(milliseconds: 750), () {
                      completer.complete(ai);
                      Navigator.of(context).pop(ai);
                    });
                  },
                );
              },
            );
            onSuccess?.call(resp);
          }
        },
      )),
    );
  }
}
