import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:get/get.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';
import 'package:motor_flutter/motor_flutter.dart';
import '../../../../mocks/controllers/mock_login_controller.dart';

void main() {
  Widget mockWidgetsTree(Widget child) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(),
        body: Center(
          child: Container(
            child: child,
          ),
        ),
      ),
    );
  }

  setUp(() {
    Get.lazyPut(() => FakeLoginController());
  });

  testWidgets('login page should render two inputs if there is domain',
      (tester) async {
    await tester.pumpWidget(
      mockWidgetsTree(
        LoginPage<FakeLoginController>(
          onSuccess: (_) {},
        ),
      ),
    );
    final sonrButtonWidgets = find.byType(SonrTextInput);
    expect(sonrButtonWidgets, findsNWidgets(2));

    // golden test
    await expectLater(
      find.byType(MaterialApp),
      matchesGoldenFile('login_page_without_domain.png'),
    );
  });

  testWidgets('login page should render only one input if there is no domain',
      (tester) async {
    await tester.pumpWidget(
      mockWidgetsTree(
        LoginPage<FakeLoginController>(
          domain: "johnsmith.snr",
          onSuccess: (_) {},
        ),
      ),
    );
    final sonrButtonWidgets = find.byType(SonrTextInput);
    expect(sonrButtonWidgets, findsNWidgets(1));

    // golden test
    await expectLater(
      find.byType(MaterialApp),
      matchesGoldenFile('login_page_with_domain.png'),
    );
  });

  testWidgets('login page user sould be able to enter text on inputs',
      (tester) async {
    await tester.pumpWidget(
      mockWidgetsTree(
        LoginPage<FakeLoginController>(
          onSuccess: (_) {},
        ),
      ),
    );

    final passwordInput = find.byKey(LoginPage.passwordFieldKey);
    await tester.enterText(passwordInput, 'Testing@1234');
    await tester.pump();
    expect(find.text('Testing@1234'), findsOneWidget);

    final domainInput = find.byKey(LoginPage.domainFieldKey);
    await tester.enterText(domainInput, 'johnsmith.snr');
    await tester.pump();
    expect(find.text('johnsmith.snr'), findsOneWidget);

    // golden test
    await expectLater(
      find.byType(MaterialApp),
      matchesGoldenFile('login_page_inputs_filled.png'),
    );
  });

  testWidgets('login page should call onSuccess after clicking "continue"',
      (tester) async {
    bool called = false;

    dynamic onSuccess(WhoIs _) {
      called = true;
    }

    await tester.pumpWidget(
      mockWidgetsTree(
        LoginPage<FakeLoginController>(
          onSuccess: onSuccess,
        ),
      ),
    );

    final continueButton = find.byKey(LoginPage.continueButtonKey);

    expect(called, false);
    await tester.tap(continueButton);
    await tester.pump();
    expect(called, true);
  });
}
